package Ubic::Catalog::Dir;
BEGIN {
  $Ubic::Catalog::Dir::VERSION = '1.07';
}

use base qw(Ubic::Multiservice::Dir);

=head1 NAME

Ubic::Catalog::Dir - deprecated alternate name of Ubic::Multiservice::Dir

=head1 VERSION

version 1.07

=cut

1;