package Ubic::Catalog;
BEGIN {
  $Ubic::Catalog::VERSION = '1.07';
}

use strict;
use warnings;

use base qw(Ubic::Multiservice);

=head1 NAME

Ubic::Catalog - deprecated alternate name of Ubic::Multiservice

=head1 VERSION

version 1.07

=cut

1;